unit Pracownik;

interface

 uses Classes, DaneOPracwoniku, FormaZatrudnienia, SysUtils;

 type
  TPracownik = class
  private
    { Pola, funkcje i procedury prywatne}
    fIloscGodzin     : real;
    fDaneOPracowniku   : TDaneOPracwoniku;
    fFormaZatrudnienia : TFormaZatrudnienia;

    function GetPlacaZasadnicza() : Real;
    function GetPremia()          : Real;
    function GetWynagrodzenie()   : Real;

    //procedury
    procedure SetIloscGodzin(const value: real);

    procedure Clear();
  public
    //waciwoci
    property IloscGodzin     : real read fIloscGodzin write SetIloscGodzin;

    //waciwoci tylko do odczytu
    property PlacaZasadnicza   : real read GetPlacaZasadnicza;
    property Premia            : real read GetPremia;
    property Wynagrodzenie     : real read GetWynagrodzenie;
    property DaneOPracowniku   : TDaneOPracwoniku read fDaneOPracowniku;
    property FormaZatrudnienia : TFormaZatrudnienia read fFormaZatrudnienia;

    //konstruktory i destruktor
    constructor Create(); overload;
    constructor Create(iloscGodzin : real; daneOPracowniku : TDaneOPracwoniku; formaZatrudnienia : TFormaZatrudnienia); overload;
    destructor Destroy(); override;

    //procedury publiczne
    procedure WriteToStream(writer : TWriter);
    procedure ReadFromStream(reader : TReader);
  end;

//stae
const iloscGodzinWMiesiacu : Integer = 170;
const stawkaGodzinowa : Integer = 15;
const stawkaZaNadgodziny : Integer = 40;


implementation

//funkcje zwracajce dane tzn liczce wynagrodzenie itd
function TPracownik.GetPlacaZasadnicza() : Real;
begin
//obliczenie stawki podstawowej
if Self.fIloscGodzin > iloscGodzinWMiesiacu then
 Result := stawkaGodzinowa * iloscGodzinWMiesiacu
else
 Result := stawkaGodzinowa * Self.fIloscGodzin;
end;

function TPracownik.GetPremia() : Real;
begin
//obliczenie premii
if Self.fIloscGodzin > iloscGodzinWMiesiacu then
 Result := (Self.fIloscGodzin - iloscGodzinWMiesiacu) * stawkaZaNadgodziny
else
 Result := 0;
end;

function TPracownik.GetWynagrodzenie() : Real;
begin
//obliczenie wynagrodzenia cakowitego (pensja zasadnicza + premia)
Result := Self.PlacaZasadnicza + Self.Premia;
end;


//procedura ustawiajaca ilo godzin
procedure TPracownik.SetIloscGodzin(const value: real);
begin
if value <> Self.fIloscGodzin then Self.fIloscGodzin := value;
end;

//procedura zapisuje zawarto obiektu pracownik do strumienia
procedure TPracownik.WriteToStream(writer : Twriter);
begin
with writer do begin
  //zapisuj znak pocztka listy
  WriteListBegin;

  //zapisuj dane
  WriteFloat(Self.fIloscGodzin);
  WriteString(Self.fDaneOPracowniku.Imie);
  WriteString(Self.fDaneOPracowniku.Nazwisko);
  WriteInteger(Self.fDaneOPracowniku.RokZatrudnienia);
  WriteInteger(Integer(Self.fFormaZatrudnienia));

  //zapisuj znak koca listy
  WriteListEnd;
end;
end;

//procedura odczytuje zawarto obiektu pracownik ze strumienia
procedure TPracownik.ReadFromStream(reader : TReader);
var imie, nazwisko : string;
    rok : Integer;
begin
with reader do begin
  //odczytuj znak pocztku listy
  ReadListBegin;

  //odczytuj dane
  Self.fIloscGodzin := ReadFloat;

  imie     := ReadString;
  nazwisko := ReadString;
  rok      := ReadInteger;

  Self.fDaneOPracowniku := TDaneOPracwoniku.Create(imie, nazwisko, rok);
  Self.fFormaZatrudnienia := TFormaZatrudnienia(ReadInteger);

  //odczytuj znak koca listy
  ReadListEnd;
end;
end;

//procedura czyszczca pola tej klasy
procedure TPracownik.Clear();
begin
Self.fIloscGodzin := 0;
end;

//konstruktory
constructor TPracownik.Create();
begin
inherited Create;

//w konstruktorze czyszcz pola
Self.Clear();
end;

constructor TPracownik.Create(iloscGodzin : real; daneOPracowniku : TDaneOPracwoniku; formaZatrudnienia : TFormaZatrudnienia);
begin
//wywoanie konstruktora bezparametrowego
Self.Create();

//ustawiam przekazane dane
Self.fIloscGodzin := iloscGodzin;
Self.fDaneOPracowniku := daneOPracowniku;
Self.fFormaZatrudnienia := formaZatrudnienia;
end;

//destruktor
destructor TPracownik.Destroy();
begin
if (Self.fDaneOPracowniku <> nil) then
 FreeAndNil(Self.fDaneOPracowniku);

inherited Destroy();
end;

end.
